@@ -17,9 +17,9 @@ use File::Basename;
 
 use Devel::PDB::Source;
 
-use vars qw(*dbline $usercontext $db_stop);
+use vars qw(*dbline $usercontext $db_stop $ini_warn);
 
-our $VERSION = '1.4';
+our $VERSION = '1.6';
 
 our $single;
 our $sub;
@@ -66,13 +66,14 @@ my $stdout;
 my $stderr;
 my $output;
 
-my $user_conf_readed = 0;
+my $user_conf_readed  = 0;
+my $ui_window_focused = 0;
 
 $trace = $signal = $single = 0;
 $stack_depth = 0;
 @stack       = (0);
 
-our %def_style = (
+my %def_style = (
     -bg  => 'white',
     -fg  => 'blue',
     -bbg => 'blue',
@@ -81,9 +82,23 @@ our %def_style = (
     -tfg => 'blue',
 );
 
+#
+# Set or return window colour style
+#
+sub window_style {
+    if (@_) {
+        my %h = @_;
+        while (my ($k, $v) = each %h) {
+            $def_style{$k} = $v if ($k =~ /^-[tbs]?[fb]g$/);
+        }
+    }
+    return %def_style;
+}
+
 BEGIN {
     $Devel::PDB::scriptName  = $0;
     @Devel::PDB::script_args = @ARGV;    # copy args
+    $ini_warn                = $^W;
 
     # This is the flag that says "a debugger is running, please call
     # DB::DB and DB::sub". We will turn it on forcibly before we try to
@@ -123,14 +138,13 @@ END {
         },
     );
 
-    my $exitloop = 0;
-    do {
-        last if ($db_exit);
+    my $exitloop = ($db_exit || !$cui) ? 1 : 0;
+    while (!$exitloop) {
         my $t = $cui->dialog(
             -title   => 'Exiting',
             -buttons => \@ab,
             -message => 'Choose one of this functions : ',
-            %def_style,
+            window_style(),
         );
 
         if ($t == 1) {
@@ -146,7 +160,8 @@ END {
             save_state_file(config_file("conf"));
             DoRestart();
         }
-    } while (!$exitloop);
+    }
+    endwin();
 }
 
 #
@@ -160,21 +175,47 @@ sub DoRestart {
     # We must destroyed $cui
     $cui = undef;
 
-    $Devel::PDB::scriptName = "" if ($Devel::PDB::scriptName =~ /perl/);
-    my @a = grep { $_ ne "-w" && $_ ne "-d:PDB" } @Devel::PDB::script_args;
+    my @flags = ();
+
+    # If warn was on before, turn it on again.
+    push @flags, '-w' if $ini_warn;
+
+    # Rebuild the -I flags that were on the initial # command line.
+    my %h_inc = @INC;
+    foreach (split(" ", `perl -e 'print "\@INC";'`)) {
+        delete($h_inc{$_});
+    }
+
+    foreach (keys %h_inc) {
+        push @flags, '-I', $_;
+    }
+
+    # Turn on taint if it was on before.
+    push @flags, '-T' if ${^TAINT};
+
+    if ($Devel::PDB::scriptName eq '-e') {
+        my $cl;
+        my $lines = *{$main::{'_<-e'}}{ARRAY};
+        for (1 .. $#$lines) {    # The first line is PERL5DB
+            chomp($cl = $lines->[$_]);
+            push @flags, '-e', $cl;
+        }
+    } elsif ($Devel::PDB::scriptName !~ /perl/) {
+        push @flags, $Devel::PDB::scriptName;
+    }
 
     # print "$$ doing a restart with $fname\n" ;
-    exec "perl", "-w", "-d:PDB", $Devel::PDB::scriptName, @a;
+    exec "perl", "-d:PDB", @flags, @Devel::PDB::script_args;
 }
 
 #
 # print any error which is put as arguments
 #
 sub print_error {
-    Devel::PDB::Dialog::Message->run(
+    $cui->error(
         -title   => "Error",
         -message => join("\n", @_),
-        %DB::def_style
+        DB::window_style(),
     ) if ($cui);
 }
 
@@ -474,8 +515,10 @@ sub ui_db_quit {
         -title   => 'Quit Debugger',
         -buttons => ['yes', 'no'],
         -message => 'Do you really want to quit?',
-        %def_style,
+        window_style(),
       );
+    save_state_file(config_file("conf.rc"));
+
     $single = 0;
     for (my $i = 0; $i <= $stack_depth; ++$i) {
         $stack[$i] = 0;
@@ -483,8 +526,6 @@ sub ui_db_quit {
 
     $db_exit = 1;
 
-    save_state_file(config_file("conf.rc"));
-
     #print(STDERR $_, "\n") foreach (@compiled);
     exit(0);
 }
@@ -538,18 +579,36 @@ sub db_toggle_break {
 # Add watch expression
 #
 sub db_add_watch_expr {
+    my $text = shift;
     my $expr = $cui->question(
         -question => "Please enter an expression to watches\n"
           . "Global variables must be set as '\$main::varname'\n"
           . 'Array or Hash must set as Reference like \@a, otherwise show size',
         -title => "Add watch expresion",
-        %def_style
+        (defined($text) && length($text) ? (-answer => $text) : ()),
+        window_style(),
     );
-    return if !$expr;
-    push @watch_exprs, {name => $expr};
+    if (defined($text) && length($text)) {
+        my $pos = -1;
+        for (my $i = 0; $pos == -1 && $i < scalar(@watch_exprs); $i++) {
+            $pos = $i if ($watch_exprs[$i]->{name} eq $text);
+        }
+        splice(@watch_exprs, $pos, 1, {name => $expr}) if ($expr && $pos >= 0);
+    } else {
+        return if !$expr;
+        push @watch_exprs, {name => $expr};
+    }
     $update_watch_list = 1;
 }
 
+sub db_edit_watch_expr {
+    my $watch_list = shift;
+
+    my $id   = $watch_list->get_active_id;
+    my $item = $watch_list->{-named_list}->[$id];
+    db_add_watch_expr($item->{name});
+}
+
 #
 # List breapoints
 #
@@ -575,7 +634,7 @@ sub ui_list_breakpoints {
         -title           => "List all breakpoints",
         -files           => \@a,
         -its_breakpoints => 1,
-        %def_style,
+        window_style(),
     );
 
     if ($filename) {
@@ -812,7 +871,7 @@ sub ui_view_stack {
         -border   => 0,
         -centered => 1,
         -title    => 'Stack',
-        %def_style,
+        window_style(),
     );
     my $listbox = $win->add(
         'StackWindow', 'Listbox',
@@ -827,7 +886,7 @@ sub ui_view_stack {
         -labels     => \%h,
 
         #-onselchange => \&on_file_active,
-        %def_style,
+        window_style(),
     );
     $win->add(
         "help", "Label",
@@ -865,6 +924,142 @@ sub ui_view_stack {
     $sv_win->focus;
 }
 
+#
+# UI export information
+#
+sub ui_db_export {
+    my $win = $cui->add(
+        'winexportwindow', 'Window',
+        -border   => 1,
+        -centered => 1,
+        -title    => 'Export information from actuall position to file',
+        -height   => 14,
+        window_style(),
+    );
+
+    $win->add("ExportLabel_1", 'Label', -y => 1, -x => 2, -text => 'Number of lines : ', -bold => 1);
+
+    my $lines = 10;
+    $win->add(
+        "ExportNumber", "TextEntry",
+        -y        => 1,
+        -x        => 20,
+        -width    => 20,
+        -text     => $lines,
+        -regexp   => '/^\d*$/',
+        -onchange => sub { $lines = shift->get(); },
+    );
+
+    $win->add("ExportLabel_2", 'Label', -y => 3, -x => 2, -text => 'Filename : ', -bold => 1);
+
+    my $filename = undef;
+    $win->add(
+        "ExportFilename", "TextEntry",
+        -y        => 3,
+        -x        => 14,
+        -width    => 30,
+        -onchange => sub { $filename = shift->get(); },
+    );
+
+    $win->add(
+        "ExportLabel_3", 'Label',
+        -y    => 5,
+        -x    => 2,
+        -text => 'Variables separated by space or export everything : ',
+        -bold => 1,
+    );
+    my $variables = undef;
+    $win->add(
+        "ExportVariables", "TextEntry",
+        -y        => 6,
+        -x        => 2,
+        -width    => 30,
+        -onchange => sub { $variables = shift->get(); },
+    );
+
+    my $use_watches = 0;
+    $win->add(
+        'ExportWatches', 'Checkbox',
+        -label    => "Export all variables from watch tables",
+        -y        => 8,
+        -x        => 2,
+        -onchange => sub { $use_watches = shift->get(); },
+        window_style(),
+    );
+
+    my $exit = 1;
+    $win->add(
+        'ExportButtons',
+        'Buttonbox',
+        -buttons => [{
+                -label    => '< Ok >',
+                -shortcut => 'o',
+                -onpress  => sub { $exit = 0; $win->loose_focus; }
+            },
+            {   -label    => '< Cancel >',
+                -shortcut => 'c',
+                -onpress  => sub {
+                    $win->loose_focus;
+                  }
+            }
+        ],
+        -y => 10,
+        -x => 2,
+    );
+
+    $win->set_binding(sub { shift->loose_focus; }, "\cQ", "\cC", KEY_F(10), CUI_ESCAPE());
+    my $sel = $win->modalfocus();
+    $cui->delete("winexportwindow");
+
+    local *W;
+    if ($exit) {
+    } elsif (!$filename || !length($filename)) {
+        print_error("Filename must be set");
+    } elsif (!open(W, ">$filename")) {
+        print_error("Can't open file $filename : $!");
+    } else {
+        local (*dbline) = $main::{'_<' . $current_source->filename};
+
+        my $current_line = $current_source->current_line;
+        my $from         = $current_line - $lines;
+        $from = 0 if ($from < 0);
+        my $to = $current_line + $lines;
+        my $l = length(sprintf("%d", $to));
+
+        print W "----- Filename : " . $current_source->filename . "----------\n";
+        for my $i ($from .. $to) {
+            last unless exists $dbline[$i];
+            if ($i == 0 && $dbline[$i] =~ /use\s+.*Devel::_?PDB/) {
+                $to++;
+                next;
+            }
+            printf W "%s%*d %s", $i == $current_line ? '*' : ' ', $l, $i, $dbline[$i];
+        }
+
+        sub print_variables {
+            my ($rh) = @_;
+            print W $rh->{name} . " -> " . $rh->{long_value} . "\n";
+        }
+        print W "----- Stack : -------------\n";
+        my %h = ();
+        %h = map { $_ => 1 } split(" ", $variables) if (length($variables));
+        foreach my $rh (@padlist_disp) {
+            print_variables($rh) if (!keys(%h) || exists($h{$rh->{name}}));
+        }
+
+        if ($use_watches) {
+            print W "----- Watches : -----------\n";
+            foreach my $rh (@watch_exprs) {
+                print_variables($rh);
+            }
+        }
+
+        close(W);
+    }
+
+    $sv_win->focus;
+}
+
 #
 # UI open file
 #
@@ -875,7 +1070,7 @@ sub ui_open_file {
         'Devel::PDB::Dialog::FileBrowser',
         -title => $title,
         -files => $files,
-        %def_style,
+        window_style(),
     );
     if ($filename) {
         my $source = $current_source = get_source($filename);
@@ -909,7 +1104,7 @@ sub db_view_std_files {
         -title   => 'Open STD* files',
         -buttons => \@ab,
         -message => 'Choose which STD* file to open it?',
-        %def_style,
+        window_style(),
     );
     return if ($t == -1);
 
@@ -925,7 +1120,7 @@ sub db_view_std_files {
         'winmytextviewer', 'Window',
         -border => 0,
         -title  => 'Source',
-        %def_style,
+        window_style(),
     );
     my $textviewer = $win->add(
         "mytextviewer", "TextViewer",
@@ -944,7 +1139,7 @@ sub db_view_std_files {
         -wrapping        => 0,       # wrapping slows down the editor :-(
         -text            => $text,
         -title => " Viewing file STD" . ($t == 2 ? "ERR" : "OUT") . " : " . config_file($t == 2 ? "stderr" : "stdout"),
-        %def_style,
+        window_style(),
     );
     $win->add(
         "help", "Label",
@@ -1022,9 +1217,8 @@ sub set_key_binding($$@) {
     my @keys = @_;
 
     if (!defined($keys_binded)) {
-        local *F;
-        if (open(F, $ENV{HOME} . "/.PDB.keys")) {
-            while (<F>) {
+        if (open(my $fh, $ENV{HOME} . "/.PDB.keys")) {
+            while (<$fh>) {
                 chomp;
                 my @a = split("=");
                 next if (scalar(@a) < 2);
@@ -1044,7 +1238,7 @@ sub set_key_binding($$@) {
                 }
                 $keys_binded->{$a[0]} = \@akeys;
             }
-            close(F);
+            close($fh);
         } else {
             $keys_binded = {};
         }
@@ -1153,7 +1347,7 @@ sub ui_text_editor {
         #-height => $row + 3,
         -centered => 1,
         -title    => $str_title,
-        %def_style,
+        window_style(),
     );
     my $x = $win->add(
         "ChangeParams", $use_editor ? "TextEditor" : "TextViewer",
@@ -1249,6 +1443,7 @@ sub ui_db_help {
     push(@a, "  ENTER\tSelect the highlighted file or apply the filter to the file list");
     push(@a, "Other");
     push(@a, "  Esc,F10\tBack,Exit function");
+    push(@a, "  Ctrl+S,Ctrl+L,F6\tExporting to file");
 
     if (keys %keys_hash) {
         my @ad = ();
@@ -1262,24 +1457,83 @@ sub ui_db_help {
         push(@a, " ", " ", "Duplicity in keys", " ", @ad) if (@ad);
     }
 
-    Devel::PDB::Dialog::Message->run(
+    dialog_message(
         -title   => "Help Keys",
         -message => join("\n", @a),
-        %DB::def_style
     );
 }
 
+#
+# Create dialog message window with binded key F2 for saving text
+#
+sub dialog_message {
+    my %args = @_;
+    Devel::PDB::Dialog::Message->run(%args, window_style());
+}
+
+#
+# Exporting to file
+#
+sub export_to_file {
+    my ($name, $title, $rh_str) = @_;
+
+    return unless $cui;
+    $name ||= "Title";
+    my $fname = $cui->question(-question => 'Add filename to export', DB::window_style()) || return;
+    if (open(my $fh, ">", $fname)) {
+        print $fh "----- $name : " . $title . " ----------\n" if ($title);
+        print $fh $$rh_str;
+        print $fh "\n";
+        close($fh);
+    } else {
+        DB::print_error("Can't open file $fname : $!");
+    }
+}
+
+#
+# Activate window
+#
+sub set_active_window {
+    my $win = shift;
+
+    if ($win == 2) {
+        $ui_window_focused = 1;
+        $auto_win->focus;
+    } elsif ($win == 3) {
+        $ui_window_focused = 2;
+
+        #ui_update_watch_list();
+        $watch_win->focus;
+    } else {
+        $ui_window_focused = 0;
+        $sv_win->focus;
+    }
+}
+
 #
 # Initialize ncurses methods
 #
 sub init {
 
+    # Set own colours
+    if (open(my $fh, $ENV{HOME} . "/.PDB.colours")) {
+        my %h;
+        while (<$fh>) {
+            chomp;
+            my @a = split(/\s+/);
+            $h{$a[0]} = $a[1];
+        }
+        close($fh);
+        window_style(%h);
+    }
+
     # can anybody tell me why $win->notimeout(1) doesn't work?
     $ENV{ESCDELAY} = '0';
 
     $cui = new Curses::UI(
         -clear_on_exit => 1,
-        -color_support => 1
+        -color_support => 1,
+        -mouse_support => 1,
     );
 
     if ($Curses::UI::VERSION > 0.9602) {
@@ -1322,7 +1576,7 @@ sub init {
         -border => 1,
 
         #-padbottom => 3,
-        %def_style,
+        window_style(),
     );
 
     $lower_win = $cui->add(
@@ -1330,7 +1584,7 @@ sub init {
         -border => 0,
         -y      => -1,
         -height => $lower_height,
-        %def_style,
+        window_style(),
     );
 
     $auto_win = $lower_win->add(
@@ -1339,7 +1593,7 @@ sub init {
         -y      => -1,
         -width  => $half_width,
         -title  => 'Auto',
-        %def_style,
+        window_style(),
     );
     $padvar_list = $auto_win->add(
         'padvar_list', 'Devel::PDB::NamedListbox',
@@ -1356,7 +1610,7 @@ sub init {
         -y       => -1,
         -padleft => $half_width,
         -title   => 'Watch',
-        %def_style,
+        window_style(),
     );
     $watch_list = $watch_win->add(
         'watch_list', 'Devel::PDB::NamedListbox',
@@ -1384,6 +1638,11 @@ sub init {
         @aFile,
         set_key_binding(
             sub {
+                if ($ui_window_focused == 2) {
+                    $update_watch_list = 1;
+                    return;
+                }
+
                 my $ret = $cui->dialog(
                     -title   => 'Restarting program',
                     -buttons => [{
@@ -1401,7 +1660,7 @@ sub init {
                         },
                     ],
                     -message => 'Choose option to restarting program',
-                    %def_style,
+                    window_style(),
                 );
                 if ($ret) {
                     save_state_file($fConfig) if ($ret == 1);
@@ -1420,7 +1679,7 @@ sub init {
                 my $filename = $cui->filebrowser(
                     -title => "Find and load Perl module from file ",
                     -mask  => [['\.p[lm]$', 'Perl modules']],
-                    %DB::def_style
+                    DB::window_style(),
                 );
                 if ($filename) {
                     if (!exists($main::{"_<$filename"})) {
@@ -1455,7 +1714,7 @@ sub init {
             sub { ui_open_file('Opened Files', [keys(%sources)]); },
             "FilesOpened", "Show 'Opened Files' Dialog",
             KEY_F(12)));
-    push(@aFile, set_key_binding(\&ui_db_quit, "Quit", "Quit the debugger", "\cQ", "\cC"));
+    push(@aFile, set_key_binding(\&ui_db_export, "Export", "Export information", "\cY"));
     push(
         @aFile,
         set_key_binding(
@@ -1469,13 +1728,27 @@ sub init {
             "Refresh windows",
             "\cN"
         ));
+    push(@aFile, set_key_binding(\&ui_db_quit, "Quit", "Quit the debugger", "\cQ", "\cC"));
 
     # Submenu - Execution
     push(@aExecution, set_key_binding(\&db_cont,      "Continue", "Run|Continue execution", KEY_F(5)));
     push(@aExecution, set_key_binding(\&db_step_out,  "StepOut",  "Step Out",               KEY_F(6)));
     push(@aExecution, set_key_binding(\&db_step_in,   "StepIn",   "Step In",                KEY_F(7)));
     push(@aExecution, set_key_binding(\&db_step_over, "StepOver", "Step Over",              KEY_F(8)));
-    push(@aExecution, set_key_binding(sub { ui_text_editor(1); }, "ArgumentsEdit",   "Edit program paramaters",    "\cE"));
+    push(
+        @aExecution,
+        set_key_binding(
+            sub {
+                if ($ui_window_focused == 2) {
+                    db_edit_watch_expr($watch_list);
+                } else {
+                    ui_text_editor(1);
+                }
+            },
+            "ArgumentsEdit",
+            "Edit program paramaters or watched variable",
+            "\cE"
+        ));
     push(@aExecution, set_key_binding(sub { ui_text_editor(2); }, "EnviromentsEdit", "Edit enviroment paramaters", "\cM"));
     push(
         @aExecution,
@@ -1484,7 +1757,7 @@ sub init {
                 my $ret = $cui->question(
                     -title    => 'Command Execution',
                     -question => 'Please enter an command to enter',
-                    %DB::def_style
+                    DB::window_style(),
                 );
                 $usercontext = $ret if ($ret);
             },
@@ -1494,9 +1767,15 @@ sub init {
         ));
 
     # Submenu - Breakpoint
-    push(@aBreakpoint, set_key_binding(sub { db_toggle_break(0, undef) }, "Breakpoint", "Toggle Breakpoint", KEY_F(9)));
-    push(@aBreakpoint, set_key_binding(sub { db_toggle_break(1, undef) }, "BreakpointCode", "Toggle Breakpoint Code", "\cO"));
-    push(@aBreakpoint, set_key_binding(\&db_add_watch_expr,   "WatchExpression", "Add watch expression", "\cW"));
+    push(@aBreakpoint,
+        set_key_binding(sub { set_active_window(1); db_toggle_break(0, undef) }, "Breakpoint", "Toggle Breakpoint", KEY_F(9)));
+    push(
+        @aBreakpoint,
+        set_key_binding(
+            sub { set_active_window(1); db_toggle_break(1, undef) },
+            "BreakpointCode", "Toggle Breakpoint Code", "\cO"
+        ));
+    push(@aBreakpoint, set_key_binding(sub { db_add_watch_expr(undef) }, "WatchExpression", "Add watch expression", "\cW"));
     push(@aBreakpoint, set_key_binding(\&ui_list_breakpoints, "ListBreakpoints", "List all breakpoints", "\cB"));
     push(@aBreakpoint, set_key_binding(\&clearalldblines, "ClearBreakpoints", "Clear all breakpoints"));
     push(@aBreakpoint,
@@ -1530,7 +1809,7 @@ sub init {
                         },
                     ],
                     -message => 'Do you really want load config?',
-                    %def_style,
+                    window_style(),
                 );
                 if ($ret) {
                     load_state_file($fConfig, ($ret == 2 ? ".rc" : ""));
@@ -1550,7 +1829,7 @@ sub init {
                     -title   => 'Save config file',
                     -buttons => ['yes', 'no'],
                     -message => 'Do you really want save config?',
-                    %def_style,
+                    window_style(),
                   );
             },
             "ConfigSave",
@@ -1569,10 +1848,9 @@ sub init {
                 local $Data::Dumper::Indent   = 2;
                 local $Data::Dumper::Sortkeys = 1;
                 $text = (scalar(@Devel::PDB::script_args) ? Dumper(@Devel::PDB::script_args) : "Not arguments putted");
-                Devel::PDB::Dialog::Message->run(
+                dialog_message(
                     -title   => "Arguments",
-                    -message => $text,
-                    %DB::def_style
+                    -message => $text
                 );
 
             },
@@ -1580,17 +1858,12 @@ sub init {
             "View program parameters",
             "\cA"
         ));
-    push(@aView, set_key_binding(sub { $sv_win->focus }, "WindowSource", "Switch to the Source Code Window", KEY_F(1)));
+    push(@aView, set_key_binding(sub { set_active_window(1) }, "WindowSource", "Switch to the Source Code Window", KEY_F(1)));
     push(@aView,
-        set_key_binding(sub { $auto_win->focus }, "WindowLexical", "Switch to the Lexical Variable Window", KEY_F(2)));
-    push(
-        @aView,
-        set_key_binding(
-            sub { ui_update_watch_list(); $watch_win->focus; },
-            "WindowWatches", "Switch to the Watch Window",
-            KEY_F(3)));
-    push(@aView, set_key_binding(sub { ui_view_stack(0) },  "WindowStack",   "View Stack Window",      "\cT"));
-    push(@aView, set_key_binding(sub { ui_text_editor(3) }, "ViewVariables", "View special variables", "\cU"));
+        set_key_binding(sub { set_active_window(2) }, "WindowLexical", "Switch to the Lexical Variable Window", KEY_F(2)));
+    push(@aView, set_key_binding(sub { set_active_window(3) }, "WindowWatches", "Switch to the Watch Window", KEY_F(3)));
+    push(@aView, set_key_binding(sub { ui_view_stack(0) },     "WindowStack",   "View Stack Window",          "\cT"));
+    push(@aView, set_key_binding(sub { ui_text_editor(3) },    "ViewVariables", "View special variables",     "\cU"));
 
     push(@aView,
         set_key_binding(sub { ui_adjust_vert_parts(1) }, "VerticalPartsMin", "Vertical window(Source file) minimize", '{'));
@@ -1627,12 +1900,14 @@ sub init {
                     },
                     {   -label => 'About',
                         -value => sub {
-                            Devel::PDB::Dialog::Message->run(
+                            dialog_message(
                                 -title   => "About",
                                 -message => <<EOF
-Devel::PDB - A simple Curses-based Perl Debugger in version $VERSION
+Devel::PDB - A simple Curses-based Perl DeBugger in version $VERSION
 
-PerlDeBugger is a Curses-based Perl debugger with most of the essential functions such as monitoring windows for paddlist, call stack, custom watch expressions, etc. Suitable for debugging or tracing complicated Perl applications on the spot.
+PerlDeBugger is a Curses-based Perl debugger with most of the essential functions such as monitoring windows for paddlist, 
+call stack, custom watch expressions, etc. 
+Suitable for debugging or tracing complicated Perl applications on the spot.
 
 AUTHORS
 Ivan Yat-Cheung Wong
@@ -1644,14 +1919,14 @@ Curses:UI - $Curses::UI::VERSION
 
 EOF
                                 ,
-                                %DB::def_style
+                                DB::window_style(),
                             );
                         },
                     },
                 ]
             },
         ],
-        %def_style,
+        window_style(),
     );
 
     #open my $fd0, '>stdout';
@@ -1763,42 +2038,52 @@ sub DB {
     # BUGS:
     # compadlist not return, not defined variables.
     # Variables must be defined via (my,our,....etc) or 'use strict;' on yours script
-    my ($names, $vals) =
-      $scope eq 'main'
-      ? comppadlist->ARRAY
-      : svref_2object(\&$scope)->PADLIST->ARRAY;
-    my @names = $names->ARRAY;
-    my @vals  = $vals->ARRAY;
-    my $count = @names;
+    {
+        my ($names, $vals) =
+          $scope eq 'main'
+          ? comppadlist->ARRAY
+          : svref_2object(\&$scope)->PADLIST->ARRAY;
+        my @names = $names->ARRAY;
+        my @vals  = $vals->ARRAY;
+        my $count = @names;
 
-    refresh_stack_menu();
+        refresh_stack_menu();
 
-    local $Data::Dumper::Terse = 1;
-    local $Data::Dumper::Maxdepth;
-    local $Data::Dumper::Indent;
-    local $Data::Dumper::Sortkeys = 1;
+        local $Data::Dumper::Terse = 1;
+        local $Data::Dumper::Maxdepth;
+        local $Data::Dumper::Indent;
+        local $Data::Dumper::Sortkeys = 1;
 
-    for (my ($i, $j) = (0, 0); $i < $count; $i++) {
-        my $sv = $names[$i];
-        next if class($sv) eq 'SPECIAL';
-        my $name = $sv->PVX;
-        $Data::Dumper::Indent   = 0;
-        $Data::Dumper::Maxdepth = 2;
-        my $val = Dumper $vals[$i]->object_2svref;
-        $val =~ s/^\\// if class($sv) ne 'RV';
-        $Data::Dumper::Indent   = 1;
-        $Data::Dumper::Maxdepth = 0;
-        my $long_val = Dumper $vals[$i]->object_2svref;
-        $long_val =~ s/^\\// if class($sv) ne 'RV';
+        my %h_pd = map { $_->{name} => $_ } @padlist_disp;
 
-        if ($renew || $val ne $padlist{$name}) {
-            $padlist_disp[$j] = {name => $name, value => $val, long_value => $long_val};
-            $padlist{$name} = $val;
+        for (my ($i, $j) = (0, 0); $i < $count; $i++) {
+            my $sv = $names[$i];
+            next if class($sv) eq 'SPECIAL';
+            my $name = $sv->PVX;
+            $Data::Dumper::Indent   = 0;
+            $Data::Dumper::Maxdepth = 2;
+            my $val = Dumper $vals[$i]->object_2svref;
+            $val =~ s/^\\// if class($sv) ne 'RV';
+            $Data::Dumper::Indent   = 1;
+            $Data::Dumper::Maxdepth = 0;
+            my $long_val = Dumper $vals[$i]->object_2svref;
+            $long_val =~ s/^\\// if class($sv) ne 'RV';
+
+            if ($renew || $val ne $padlist{$name}) {
+                my $rh = {name => $name, value => $val, long_value => $long_val};
+                $padlist_disp[$j] = $rh;
+                $padlist{$name}   = $val;
+                $h_pd{$name}      = $rh;
+            }
+            ++$j;
         }
-        ++$j;
-    }
 
-    $padvar_list->update($renew);
+        # Sorting values in stack by name
+        @padlist_disp = ();
+        @padlist_disp = sort { $a->{name} cmp $b->{name} } values %h_pd;
+
+        $padvar_list->update($renew);
+    }
 
     #local (*dbline) = $main::{"_<$filename"};
     $sv->source($current_source = get_source($filename));
@@ -1810,8 +2095,7 @@ sub DB {
 
     # Breakpoint with action
     my $brkp = $current_source->ret_line_breakpoint();
-    my ($stop, $action);
-    ($stop, $action) = split(/\0/, $brkp) if ($brkp);
+    my ($stop, $action) = $brkp ? split(/\0/, $brkp) : ();
     if ($action) {
         my $res = eval "return 1 if ($action); return 0;\n";
         if ($@) {
@@ -1902,7 +2186,7 @@ __END__
 
 =head1 NAME
 
-Devel::PDB - A simple Curses-based Perl Debugger
+Devel::PDB - A simple Curses-based Perl DeBugger
 
 =head1 SYNOPSIS
 
@@ -1912,8 +2196,8 @@ Devel::PDB - A simple Curses-based Perl Debugger
 
 PerlDeBugger is a Curses-based Perl debugger with most of the
 essential functions such as monitoring windows for paddlist,
-call stack, custom watch expressions, etc. Suitable for debugging or
-tracing complicated Perl applications on the spot.
+call stack, custom watch expressions, etc. 
+Suitable for debugging or tracing complicated Perl applications on the spot.
 
 =begin html
 
@@ -1934,7 +2218,7 @@ table.screen tr td, .normal {
 }
 </style>
 <table class="screen" cellspacing="0" cellpadding="0">
-<tr><td>&nbsp;&nbsp;File&nbsp;&nbsp;Help&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td></tr>
+<tr><td>&nbsp;&nbsp;File&nbsp;&nbsp;View&nbsp;&nbsp;Execution&nbsp;&nbsp;Breakpoint&nbsp;&nbsp;Settings&nbsp;&nbsp;Help&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td></tr>
 <tr><td><span class="border">&nbsp;<span class="caption">&nbsp;a.pl:5&nbsp;</span>----------------------------------------------------------------------&nbsp;</span></td></tr>
 <tr><td><span class="border">|</span>&nbsp;&nbsp;use&nbsp;Devel::PDB;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class="border">|</span></td></tr>
 <tr><td><span class="border">|</span>&nbsp;&nbsp;#!/usr/bin/perl&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class="border">|</span></td></tr>
@@ -1964,8 +2248,7 @@ table.screen tr td, .normal {
 
 =head2 FUNCTIONS
 
-PerlDeBugger is still in development stage, not all of the planed
-functions have been implemented yet. Currently it can:
+PerlDeBugger currently can:
 
 =over
 
@@ -1979,6 +2262,10 @@ set/remove breakpoint
 
 =item *
 
+Evaluate breakpoint
+
+=item *
+
 automatic display of lexical variables
 
 =item *
@@ -1997,6 +2284,10 @@ Stack Trace Window
 
 Immediate Window for executing arbitrary perl statement
 
+=item *
+
+Other functionalities
+
 =back
 
 =head2 KYES BINDING - standart key
@@ -2067,6 +2358,10 @@ Add/Edit/Remove breakpoint with condition on given line. Can be also removed by
 
 Refresh all window contents
 
+=item Export - Ctrl+Y
+
+Export information to file from actual source and stack variables or watches
+
 =item WatchExpression - Ctrl+W
 
 Add watch expression
@@ -2205,6 +2500,10 @@ Toggle the focus between the file list and the filter
 
 Select the highlighted file or apply the filter to the file list
 
+=item F6, Ctrl+S, Ctrl+L
+
+Export everything from window to given file
+
 =back
 
 =back
@@ -2229,12 +2528,22 @@ Output standart STD files from runned program
 
 Configuration files of rebinded keys. 
 For function keys is FX and for Cotrol keys is Control-X.
+
 For example keys 'F10' for open Menu and keys 'Ctrl+C','Ctrl+Q','Q' for Quit.
 
 Menu=F10
-
 Quit=Control-C,Control-Q,Q
 
+=item ~/.PDB.colours
+
+Configuration of own colours as defined in Curses::UI::Color.
+Each line has one definition, where frst is key and second is colour with space separattor.
+
+For example set general foreground and background color as RED on WHITE:
+
+-fg red
+-bg white
+
 =back
 
 =head1 SEE ALSO
@@ -2243,13 +2552,13 @@ L<perldebug>
 
 =head1 AUTHORS
 
-Ivan Yat-Cheung Wong E<lt>email@ivanwong.infoE<gt>
+Ivan Yat-Cheung Wong E<lt>email (at) ivanwong.infoE<gt>
 
 Igor Bujna E<lt>igor.bujna (at) post.czE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2007 by Ivan Y.C. Wong
+Copyright (C) 2007 by Ivan Y.C. Wong, Igor Bujna
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.8 or,
